#include "tcp_client.hpp"
#include "abstract_tcp_connect.hpp"
#include "client_tcp_connect.hpp"
#include "coroutine.hpp"
#include "error_code.hpp"
#include "fd_event.hpp"
#include "logger.hpp"
#include "net_address.hpp"
#include "reactor.hpp"
#include "tinypb_coder.hpp"
#include <unistd.h>


namespace tinyrpc {

    TcpClient::TcpClient(const NetAddress& peerAddr) : 
        m_local_addr("127.0.0.1", 0),
        m_peer_addr(peerAddr),
        m_reactor(*Reactor::getReactor())
        
    {
        m_fd = socket(AF_INET, SOCK_STREAM, 0);
        if (m_fd == -1) {
            logger() << "call socket error, fd=-1, sys error=" << strerror(errno);
        }

        m_coder = new TinypbCoder();
        // m_connection = new ClientTcpConnection(m_fd, m_reactor, *this);

    }

    TcpClient::~TcpClient() {
        logger() << "~TcpClient";
        m_reactor.delFdEvent(FdEventPool::getInstance()->getFdEvent(m_fd));
        // if(m_fd != -1) close(m_fd);
        delete m_coder;
    }
    bool TcpClient::connectToServer() {
        if(m_connection.get() == nullptr || m_connection->getState() == ClientTcpConnection::State::Disconnected) {
            int ret = connect(m_fd, m_peer_addr.getSockaddr(), m_peer_addr.getSockLen());
            if(ret == -1) return false;
            m_connection.reset(new ClientTcpConnection(m_fd, m_reactor, *this));
        }
        return true;
    }

    bool TcpClient::writeToSendBuffer(const AbstractData& data) {
        connectToServer();
        return m_coder->encoder(m_connection->getSendBuffer(), data);
    }

    int TcpClient::sendAndRecvData(const std::string& msg_req, std::shared_ptr<AbstractData>& res) {

        if(!connectToServer()) {
            logger() << "error1";
            return ERROR_FAILED_CONNECT;
        }
        m_connection->output();

        if(m_connection->getState() == ClientTcpConnection::State::Disconnected) {
            logger() << "error1";
            return ERROR_FAILED_GET_REPLY;
        }

        do {
            m_connection->input();
            m_connection->process();
            if(m_connection->getState() != ClientTcpConnection::State::Connected) {
                logger() << "error1";
                return ERROR_FAILED_GET_REPLY;
            }
        }while(!m_connection->getResPackageData(msg_req, res));

        return 0;
    }


    // void TcpClient::addCoroutine(Coroutine& cor) {

    //     Reactor::Task task = [&cor] {
    //         cor.resume();
    //     };
    //     m_reactor.addTask(task);
    // }

    // void TcpClient::start() {
    //     Coroutine::getMainCoroutine();
    //     m_reactor.loop();
    // }
}